class InvalidCommandException(Exception):
    def __init__(self,*args,**kwargs):
        Exception.__init__(self,*args,**kwargs)

class InvalidUsageException(Exception):
    def __init__(self,*args,**kwargs):
        Exception.__init__(self,*args,**kwargs)

class InvalidFormatException(Exception):
    def __init__(self,*args,**kwargs):
        Exception.__init__(self,*args,**kwargs)

class FileNotFoundException(Exception):
    def __init__(self,*args,**kwargs):
        Exception.__init__(self,*args,**kwargs)

class UnknownContentKindError(Exception):
    def __init__(self,*args,**kwargs):
        Exception.__init__(self,*args,**kwargs)

class UnknownQuestionTypeError(Exception):
    def __init__(self,*args,**kwargs):
        Exception.__init__(self,*args,**kwargs)

class InvalidInputAnswerException(Exception):
    def __init__(self,*args,**kwargs):
        Exception.__init__(self,*args,**kwargs)
def raise_for_invalid_channel(channel):
	pass


from ricecooker.classes import *
from ricecooker.managers import ChannelManager

def uploadchannel(path, domain, verbose=False):
    """ guess_content_kind: determines what kind the content is
        Args:
            path (str): path to file containing channel data
            verbose (bool): indicates whether to print process
        Returns: (str) link to access newly created channel
    """
    exec(open(path).read(), globals())

    if verbose:
        print("\n\n***** Starting channel build process *****")
        print("Constructing channel...")
    channel = construct_channel({}) # Create channel (using method from imported file)

    if verbose:
        print("Setting up initial channel structure...")
    tree = ChannelManager(channel, domain, verbose) # Create channel manager with channel data

    if verbose:
        print("Processing content...")
    tree.process_tree(channel) # Fill in values necessary for next steps

    if verbose:
        print("Getting file diff...")
    file_diff = tree.get_file_diff() # Determine which files have not yet been uploaded to the CC server

    if verbose:
        print("Uploading {0} new file(s) to the content curation server...".format(len(file_diff)))
    tree.upload_files(file_diff) # Upload new files to CC

    if verbose:
        print("Creating tree on the content curation server...")
    channel_link = tree.upload_tree() # Create tree

    print("DONE: Channel created at {0}".format(channel_link))
    return channel_link


"""Usage: ricecooker uploadchannel [-hvqr] <file_path> [--debug]

Arguments:
  file_path        Path to file with channel data

Options:
  -h --help
  -v       verbose mode
  -q       quiet mode
  -r       make report
"""

from ricecooker.commands import uploadchannel
from ricecooker import config
from docopt import docopt

commands = ["uploadchannel"]

if __name__ == '__main__':
    arguments = docopt(__doc__)
    domain = config.PRODUCTION_DOMAIN
    if arguments["--debug"]:
    	domain = config.DEBUG_DOMAIN
    uploadchannel(arguments["<file_path>"], domain, verbose=arguments["-v"])


import uuid
import hashlib
import json
import requests
import tempfile
import shutil
import os
from ricecooker import classes, config
from ricecooker.exceptions import InvalidFormatException

""" ChannelManager: used to process channel and communicate to content curation server
    @param channel (Channel to process)
    @param verbose (boolean indicating whether to print statements)
"""
class ChannelManager:
    def __init__(self, channel, domain, verbose=False):
        self.channel = channel # Channel to process
        self.verbose = verbose # Determines whether to print process
        self.domain = domain # Domain to upload channel to
        self._file_mapping = {} # Used to keep track of files and their respective metadata


    """ process_tree: sets ids and processes files
        @param node (node to process)
        @param parent (parent of node being processed)
        @return None
    """
    def process_tree(self, node, parent=None):
        if not isinstance(node, classes.Channel):
            node.set_ids(self.channel._internal_domain, parent.node_id)
            node.files = self.download_files(node.files)
        for child_node in node.children:
            self.process_tree(child_node, node)


    """ download_files: downloads files to local storage
        @param files (list of files to download)
        @return list of file hashes and extensions
    """
    def download_files(self,files):
        hash = hashlib.md5()
        hashes = [] # list of downloaded files (format:[{hash}.{ext}, {hash}.{ext}...]

        for f in files:
            if self.verbose:
                print("\tDownloading {0}...".format(f.split('/')[-1]))

            # Try to access file
            r = requests.get(f, stream=True)
            r.raise_for_status()

            # Write file to temporary file
            with tempfile.TemporaryFile() as tempf:
                for chunk in r:
                    hash.update(chunk)
                    tempf.write(chunk)

                # Get file metadata (hashed filename, original filename, size)
                hashstring = hash.hexdigest()
                original_filename = f.split("/")[-1].split(".")[0]
                filename = '{0}{ext}'.format(hashstring, ext=os.path.splitext(f)[-1])
                file_size = tempf.tell()
                tempf.seek(0)

                # Add file to list of files
                hashes += [filename]

                # Write file to local storage
                with open(filename, 'wb') as destf:
                    self._file_mapping.update({filename : {'original_filename': original_filename, 'source_url': f, 'size': file_size}})
                    shutil.copyfileobj(tempf, destf)
        return hashes


    """ get_file_diff: retrieves list of files that do not exist on content curation server
        @return list of files that are not on content curation server
    """
    def get_file_diff(self):
        response = requests.post(config.file_diff_url(self.domain), data=json.dumps([key for key, value in self._file_mapping.items()]))
        return json.loads(response._content.decode("utf-8"))


    """ upload_files: uploads files to content curation server
        @param file_list (list of files to upload)
        @return None
    """
    def upload_files(self, file_list):
        for f in file_list:
            with  open(f, 'rb') as file_obj:
                response = requests.post(config.file_upload_url(self.domain), files={'file': file_obj})
                response.raise_for_status()


    """ upload_tree: sends processed channel data to content curation to create tree
        @return link to open newly created channel
    """
    def upload_tree(self):
        payload = {
            "channel_data":self.channel.to_dict(),
            "content_data": [child.to_dict() for child in self.channel.children],
            "file_data": self._file_mapping,
        }
        response = requests.post(config.create_channel_url(self.domain), data=json.dumps(payload))
        response.raise_for_status()
        new_channel = json.loads(response._content.decode("utf-8"))
        return config.open_channel_url(new_channel['invite_id'], new_channel['new_channel'], self.domain)


from ricecooker.classes import Channel, Video, Audio, Document, Topic, guess_content_kind
from ricecooker.exceptions import UnknownContentKindError, UnknownQuestionTypeError, raise_for_invalid_channel
from le_utils.constants import content_kinds,file_formats, format_presets, licenses, exercises

SAMPLE_TREE = [
    {
        "title": "Western Philosophy",
        "id": "abd115",
        "description": "Philosophy materials for the budding mind.",
        "children": [
            {
                "title": "Nicomachean Ethics",
                "id": "ffda92",
                "author": "Aristotle",
                "description": "The Nicomachean Ethics is the name normally given to ...",
                # "file": ["https://archive.org/download/petersethics00arisrich/petersethics00arisrich.pdf"],
                "license": licenses.PUBLIC_DOMAIN,
            },
            {

                "title": "The Critique of Pure Reason",
                "id": "6ef99c",
                "description": "Kant saw the Critique of Pure Reason as an attempt to bridge the gap...",
                "children": [
                    {
                        "title": "01 - The Critique of Pure Reason",
                        "id": "8326cc",
                        "related_to": ["aaaa4d"],
                        "file": "https://archive.org/download/critique_pure_reason_0709_librivox/critique_of_pure_reason_01_kant.mp3",
                        # "subtitle": "https://archive.org/download/critique_pure_reason_0709_librivox/critique_of_pure_reason_01_kant.vtt",
                        "author": "Immanuel Kant",
                        "license": licenses.PUBLIC_DOMAIN,
                    },
                    {
                        "title": "02 - Preface to the Second Edition",
                        "id": "aaaa4d",
                        "author": "Immanuel Kant",
                        # "file": "https://ia801406.us.archive.org/13/items/alice_in_wonderland_librivox/wonderland_ch_01.mp3",
                        "author": "Immanuel Kant",
                        "license": licenses.PUBLIC_DOMAIN,
                    }
                ]
            },
        ]
    },
    {
        "title": "Recipes",
        "id": "d98752",
        "description": "Recipes for various dishes.",
        "children": [
            {
                "title": "Smoked Brisket Recipe",
                "id": "418799",
                "author": "Bradley Smoker",
                # "file": "https://archive.org/download/SmokedBrisketRecipe/smokedbrisketrecipebybradleysmoker.mp4",
                "subtitle": "something.vtt",
                "license": licenses.CC_BY,
            },
            {
                "title": "Food Mob Bites 10: Garlic Bread",
                "id": "6cafe2",
                "author": "Revision 3",
                "description": "Basic garlic bread recipe.",
                # "file": "https://archive.org/download/Food_Mob_Bites_10/foodmob--bites--0010--garlicbread--hd720p30.h264.mp4",
                "license": licenses.CC_BY_NC_SA,
            },
            {
                "title": "Recipe Exercise",
                "id": "6cafe1",
                "description": "Test how well you know your recipes",
                "license": licenses.CC_BY_NC_SA,
                "mastery_model": exercises.SKILL_CHECK,
                "questions": [
                    {
                        "id": "eeeee",
                        "question": "Which rice is your favorite?",
                        "type":exercises.MULTIPLE_SELECTION,
                        "correct_answers": ["White rice", "Brown rice", "Sushi rice"],
                        "all_answers": ["White rice", "Quinoa","Brown rice"],
                        "hint": "",
                        "images": ["file://blah/somewhere.jpg"],
                    },
                    {
                        "id": "bbbbb",
                        "question": "Which rice is the crunchiest?",
                        "type":exercises.SINGLE_SELECTION,
                        "correct_answer": "Rice Krispies",
                        "all_answers": ["White rice", "Brown rice", "Rice Krispies"],
                        "hint": "Has rice in it",
                        "images": [],
                    },
                    {
                        "id": "ccccc",
                        "question": "Why a rice cooker?",
                        "type":exercises.FREE_RESPONSE,
                        "answers": [],
                        "hint": "",
                        "images": [],
                    },
                    {
                        "id": "aaaaa",
                        "question": "How many minutes does it take to cook rice?",
                        "type":exercises.INPUT_QUESTION,
                        "answers": [20, "20.5", 19.5],
                        "hint": "Takes roughly same amount of time to install kolibri on Windows machine",
                        "images": [],
                    },
                ],
            },
        ]
    },
]

def construct_channel(args):

    channel = Channel(
        domain="learningequality.org",
        channel_id="sample-channel",
        title="Sample channel",
        thumbnail="https://s.graphiq.com/sites/default/files/stories/t4/15_Tiniest_Dog_Breeds_1718_3083.jpg",
    )
    _build_tree(channel, SAMPLE_TREE)
    raise_for_invalid_channel(channel)

    return channel


def _build_tree(node, sourcetree):

    for child_source_node in sourcetree:
        try:
            kind = guess_content_kind(child_source_node.get("file"), child_source_node.get("questions"))
        except UnknownContentKindError:
            continue

        if kind == content_kinds.TOPIC:
            child_node = Topic(
                id=child_source_node["id"],
                title=child_source_node["title"],
                author=child_source_node.get("author"),
                description=child_source_node.get("description"),
            )
            node.add_child(child_node)

            source_tree_children = child_source_node.get("children", [])

            _build_tree(child_node, source_tree_children)

        elif kind == content_kinds.VIDEO:

            child_node = Video(
                id=child_source_node["id"],
                title=child_source_node["title"],
                author=child_source_node.get("author"),
                description=child_source_node.get("description"),
                files=child_source_node.get("file"),
                license=child_source_node.get("license"),

                # video-specific data
                preset=format_presets.VIDEO_HIGH_RES,
                transcode_to_lower_resolutions=True,
                derive_thumbnail=True,

                # audio and video shared data
                subtitle=child_source_node.get("subtitle"),
            )
            node.add_child(child_node)

        elif kind == content_kinds.AUDIO:
            child_node = Audio(
                id=child_source_node["id"],
                title=child_source_node["title"],
                author=child_source_node.get("author"),
                description=child_source_node.get("description"),
                files=child_source_node.get("file"),
                license=child_source_node.get("license"),

                # audio and video shared data
                subtitle=child_source_node.get("subtitle"),
            )
            node.add_child(child_node)

        elif kind == content_kinds.DOCUMENT:
            child_node = Document(
                id=child_source_node["id"],
                title=child_source_node["title"],
                author=child_source_node.get("author"),
                description=child_source_node.get("description"),
                files=child_source_node.get("file"),
                license=child_source_node.get("license"),
            )
            node.add_child(child_node)

        elif kind == content_kinds.EXERCISE:
            child_node = Exercise(
                id=child_source_node["id"],
                title=child_source_node["title"],
                author=child_source_node.get("author"),
                description=child_source_node.get("description"),
                files=child_source_node.get("file"),
                exercise_data={'mastery_model': child_source_node.get("mastery_model"), 'randomize': True},
                license=child_source_node.get("license"),
            )
            for q in child_source_node.get("questions"):
                question = create_question(q)
                child_node.add_question(question)
            node.add_child(child_node)

        else:                   # unknown content file format
            continue

    return node

def create_question(raw_question):

    if raw_question["type"] == exercises.MULTIPLE_SELECTION:
        return MultipleSelectQuestion(
            id=raw_question["id"],
            question=raw_question["question"],
            correct_answers=raw_question["correct_answers"],
            all_answers=raw_question["all_answers"],
            hint=raw_question["hint"],
            images=raw_question["images"],
        )
    if raw_question["type"] == exercises.SINGLE_SELECTION:
        return SingleSelectQuestion(
            id=raw_question["id"],
            question=raw_question["question"],
            correct_answer=raw_question["correct_answer"],
            all_answers=raw_question["all_answers"],
            hint=raw_question["hint"],
            images=raw_question["images"],
        )
    if raw_question["type"] == exercises.INPUT_QUESTION:
        return InputQuestion(
            id=raw_question["id"],
            question=raw_question["question"],
            answers=raw_question["answers"],
            hint=raw_question["hint"],
            images=raw_question["images"],
        )
    if raw_question["type"] == exercises.FREE_RESPONSE:
        return FreeResponseQuestion(
            id=raw_question["id"],
            question=raw_question["question"],
            hint=raw_question["hint"],
            images=raw_question["images"],
        )
    else:
        raise UnknownQuestionTypeError("Unrecognized question type '{0}': accepted types are {1}".format(raw_question["type"], [key for key, value in exercises.question_choices]))


import uuid
import hashlib
import base64
import requests
import validators
import json
import tempfile
from PIL import Image
from io import BytesIO
from le_utils.constants import content_kinds,file_formats, format_presets, licenses, exercises
from ricecooker.exceptions import UnknownQuestionTypeError, InvalidInputAnswerException

def guess_content_kind(files, questions=[]):
    """ guess_content_kind: determines what kind the content is
        Args:
            files (str or list): files associated with content
        Returns: string indicating node's kind
    """
    files = [files] if isinstance(files, str) else files
    questions=[questions] if isinstance(questions, str) else questions
    if files is not None and len(files) > 0:
        for f in files:
            ext = f.rsplit('/', 1)[-1].split(".")[-1].lower()
            if ext in content_kinds.MAPPING:
                return content_kinds.MAPPING[ext]
        raise InvalidFormatException("Invalid file type: Allowed formats are {0}".format([key for key, value in content_kinds.MAPPING.items()]))
    elif questions is not None and len(questions) > 0:
        return content_kinds.EXERCISE
    else:
        return content_kinds.TOPIC

""" TreeModel: model to handle structure of channel """
class TreeModel:
    def __init__(self):
        self.children = []

    """ to_dict: formats data to what CC expects
        @return dict of model's data
    """
    def to_dict(self):
        pass

    """ add_child: adds children to root node
        @param node (node to add to children)
    """
    def add_child(self, node):
        self.children += [node]


class Channel(TreeModel):
    """ Model representing the channel you are creating

        Used to store metadata on channel that is being created

        Attributes:
            channel_id (str): channel's unique id
            domain (str): who is providing the content (e.g. learningequality.org)
            title (str): name of channel
            thumbnail (str): file path or url of channel's thumbnail
            description (str): description of the channel
    """
    def __init__(self, channel_id, domain=None, title=None, thumbnail=None, description=None):
        self.domain = domain
        self.id = uuid.uuid3(uuid.NAMESPACE_DNS, uuid.uuid5(uuid.NAMESPACE_DNS, channel_id).hex)
        self.name = title
        self.thumbnail = self.encode_thumbnail(thumbnail)
        self.description = description

        # Add data to be used in next steps
        self._internal_domain = uuid.uuid5(uuid.NAMESPACE_DNS, self.domain)
        self.content_id = uuid.uuid5(self._internal_domain, self.id.hex)
        self.node_id = uuid.uuid5(self.id, self.content_id.hex)
        super(Channel, self).__init__()


    def to_dict(self):
        """ to_dict: puts data in format CC expects
            Args: None
            Returns: dict of channel data
        """
        return {
            "id": self.id.hex,
            "name": self.name,
            "has_changed": True,
            "thumbnail": self.thumbnail,
            "description": self.description if self.description is not None else "",
            "children": [child_node.to_dict() for child_node in self.children],
        }


    def encode_thumbnail(self, thumbnail):
        """ encode_thumbnail: gets base64 encoding of thumbnail
            Args:
                thumbnail (str): file path or url to channel's thumbnail
            Returns: base64 encoding of thumbnail
        """
        if thumbnail is None:
            return None
        else:
            if validators.url(thumbnail):
                r = requests.get(thumbnail, stream=True)
                if r.status_code == 200:
                    thumbnail = tempfile.TemporaryFile()
                    for chunk in r:
                        thumbnail.write(chunk)

            img = Image.open(thumbnail)
            width = 200
            height = int((float(img.size[1])*float(width/float(img.size[0]))))
            img.thumbnail((width,height), Image.ANTIALIAS)
            bufferstream = BytesIO()
            img.save(bufferstream, format="PNG")
            return "data:image/png;base64," + base64.b64encode(bufferstream.getvalue()).decode('utf-8')



class Node(TreeModel):
    """ Model representing the nodes in the channel's tree

        Base model for different content node kinds (topic, video, exercise, etc.)

        Attributes:
            id (str): content's original id
            title (str): content's title
            description (str): description of content
            author (str): who created the content
            license (str): content's license (using constants from fle_utils)
            files (str or list): content's associated file(s)
    """
    def __init__(self, id, title, description, author, license=None, files=[], questions=[], extra_fields={}):
        self.id = id
        self.title = title
        self.description = description
        self.author = author
        self.license = license
        self.children = []
        self.files = [files] if isinstance(files, str) else files
        self.questions=questions
        self.extra_fields = extra_fields
        super(Node, self).__init__()


    def to_dict(self):
        """ to_dict: puts data in format CC expects
            Args: None
            Returns: dict of node's data
        """
        return {
            "title": self.title,
            "description": self.description if self.description is not None else "",
            "node_id": self.node_id.hex,
            "content_id": self.content_id.hex,
            "author": self.author if self.author is not None else "",
            "children": [child_node.to_dict() for child_node in self.children],
            "files" : self.files,
            "kind": self.kind,
            "license": self.license,
            "questions": self.questions,
            "extra_fields": json.dumps(self.extra_fields),
        }

    def set_ids(self, domain, parent_id):
        """ set_ids: sets ids to be used in building tree
            Args:
                domain (uuid): uuid of channel domain
                parent_id (uuid): parent node's node_id
            Returns: None
        """
        self.content_id = uuid.uuid5(domain, self.id)
        self.node_id = uuid.uuid5(parent_id, self.content_id.hex)



class Topic(Node):
    """ Model representing channel topics

        Topic nodes are used to add organization to the channel's content

        Attributes:
            id (str): content's original id
            title (str): content's title
            description (str): description of content
            author (str): who created the content
    """
    def __init__(self, id, title, description=None, author=None):
        self.kind = content_kinds.TOPIC
        super(Topic, self).__init__(id, title, description, author)



class Video(Node):
    """ Model representing videos in channel

        Videos must be mp4 format

        Attributes:
            id (str): content's original id
            title (str): content's title
            author (str): who created the content
            description (str): description of content
            transcode_to_lower_resolutions (bool): indicates whether to extract lower resolution
            derive_thumbnail (bool): indicates whether to derive thumbnail from video
            preset (str): default preset for files
            subtitle (str): path or url to file's subtitles
            license (str): content's license (using constants from fle_utils)
            files (str or list): content's associated file(s)
    """
    default_preset = format_presets.VIDEO_HIGH_RES
    def __init__(self, id, title, author=None, description=None, transcode_to_lower_resolutions=False, derive_thumbnail=False, license=None, subtitle=None, files=[], preset=None):
        if preset is not None:
            self.default_preset = preset
        if transcode_to_lower_resolutions:
            self.transcode_to_lower_resolutions()
        if derive_thumbnail:
            self.derive_thumbnail()
        self.kind = content_kinds.VIDEO
        super(Video, self).__init__(id, title, description, author, license, files)

    def derive_thumbnail(self):
        """ derive_thumbnail: derive video's thumbnail
            Args: None
            Returns: None
        """
        pass

    def transcode_to_lower_resolutions(self):
        """ transcode_to_lower_resolutions: transcode video to lower resolution
            Args: None
            Returns: None
        """
        pass



class Audio(Node):
    """ Model representing audio content in channel

        Audio can be in either mp3 or wav format

        Attributes:
            id (str): content's original id
            title (str): content's title
            author (str): who created the content
            description (str): description of content
            subtitle (str): path or url to file's subtitles
            license (str): content's license (using constants from fle_utils)
            files (str or list): content's associated file(s)
    """
    default_preset = format_presets.AUDIO
    def __init__(self, id, title, author=None, description=None, license=None, subtitle=None, files=[]):
        self.kind = content_kinds.AUDIO
        super(Audio, self).__init__(id, title, description, author, license, files)



class Document(Node):
    """ Model representing documents in channel

        Documents must be pdf format

        Attributes:
            id (str): content's original id
            title (str): content's title
            author (str): who created the content
            description (str): description of content
            license (str): content's license (using constants from fle_utils)
            files (str or list): content's associated file(s)
    """
    default_preset = format_presets.DOCUMENT
    def __init__(self, id, title, author=None, description=None, license=None, files=[]):
        self.kind = content_kinds.DOCUMENT
        super(Document, self).__init__(id, title, description, author, license, files)



class Exercise(Node):
    """ Model representing exercises in channel

        Exercises are sets of questions to assess learners'
        understanding of the content

        Attributes:
            id (str): content's original id
            title (str): content's title
            author (str): who created the content
            description (str): description of content
            license (str): content's license (using constants from fle_utils)
            files (str or list): content's associated file(s)
    """
    default_preset = format_presets.EXERCISE
    def __init__(self, id, title, author=None, description=None, license=None, files=None, exercise_data=None):
        self.kind = content_kinds.EXERCISE
        self.questions = []
        files = [] if files is None else files
        exercise_data = {} if exercise_data is None else exercise_data
        super(Exercise, self).__init__(id, title, description, author, license, files, self.questions, exercise_data)

    def add_question(self, question):
        self.questions += [question.to_dict()]

class PerseusExercise(Exercise):
    """ Model representing exercises in channel

        Exercises that are in perseus format

        Attributes:
            id (str): content's original id
            title (str): content's title
            author (str): who created the content
            description (str): description of content
            license (str): content's license (using constants from fle_utils)
            files (str or list): content's associated file(s)
    """
    default_preset = format_presets.EXERCISE
    def __init__(self, id, title, author=None, description=None, license=None, files=None, exercise_data=None):
        super(PerseusExercise, self).__init__(id, title, description, author, license, files, exercise_data)

class BaseQuestion:
    """ Base model representing exercise questions

        Questions are used to assess learner's understanding

        Attributes:
            id (str): question's unique id
            question (str): question text
            question_type (str): what kind of question is this
            answers ([{'answer':str, 'correct':bool, 'hint':str}]): answers to question
            hint (str): optional hint on how to answer question
            images ([str]): list of paths to images in question
    """
    def __init__(self, id, question, question_type, answers, hint="", images=None):
        self.question = question
        self.question_type = question_type
        self.answers = answers
        self.hint = hint
        self.images = [] if images is None else images
        self.id = uuid.uuid5(uuid.NAMESPACE_DNS, id)

    def to_dict(self):
        """ to_dict: puts data in format CC expects
            Args: None
            Returns: dict of node's data
        """
        return {
            "assessment_id": self.id.hex,
            "type": self.question_type,
            "question": self.question,
            "help_text": self.hint if self.hint is not None else "",
            "answers": json.dumps(self.answers),
        }

    def create_answer(self, answer, correct=True):
        return {"answer": answer, "correct":correct}

class MultipleSelectQuestion(BaseQuestion):
    """ Model representing multiple select questions

        Multiple select questions have a set of answers for
        the learner to select. There can be multiple answers for
        a question (e.g. Which of the following are prime numbers?
        A. 1, B. 2, C. 3, D. 4)

        Attributes:
            id (str): question's unique id
            question (str): question text
            answers ([{'answer':str, 'correct':bool, 'hint':str}]): answers to question
            hint (str): optional hint on how to answer question
            images ([str]): list of paths to images in question
    """

    def __init__(self, id, question, correct_answers, all_answers, hint="", images=None):
        set_all_answers = set(all_answers)
        all_answers += [answer for answer in correct_answers if answer not in set_all_answers]
        answers = [self.create_answer(answer, answer in correct_answers) for answer in all_answers]
        super(MultipleSelectQuestion, self).__init__(id, question, exercises.MULTIPLE_SELECTION, answers, hint, images)

class SingleSelectQuestion(BaseQuestion):
    """ Model representing single select questions

        Single select questions have a set of answers for
        with only one correct answer. (e.g. How many degrees are in a right angle?
        A. 45, B. 90, C. 180, D. None of the above)

        Attributes:
            id (str): question's unique id
            question (str): question text
            answers ([{'answer':str, 'correct':bool, 'hint':str}]): answers to question
            hint (str): optional hint on how to answer question
            images ([str]): list of paths to images in question
    """
    def __init__(self, id, question, correct_answer, all_answers, hint="", images=None):
        if correct_answer not in all_answers:
            all_answers += [correct_answer]
        answers = [self.create_answer(answer, answer==correct_answer) for answer in all_answers]
        super(SingleSelectQuestion, self).__init__(id, question, exercises.SINGLE_SELECTION, answers, hint, images)

class FreeResponseQuestion(BaseQuestion):
    """ Model representing free response questions

        Free response questions are open-ended questions
        that have no set answer (e.g. Prove that the sum of
        every triangle's angles is 360 degrees.)

        Attributes:
            id (str): question's unique id
            question (str): question text
            hint (str): optional hint on how to answer question
            images ([str]): list of paths to images in question
    """
    def __init__(self, id, question, hint="", images=None):
        super(FreeResponseQuestion, self).__init__(id, question, exercises.FREE_RESPONSE, [], hint, images)

class InputQuestion(BaseQuestion):
    """ Model representing input questions

        Input questions are questions that have one or more numerical
        answers (e.g. Name a factor of 10. ____)

        Attributes:
            id (str): question's unique id
            question (str): question text
            answers ([{'answer':str, 'hint':str}]): answers to question
            hint (str): optional hint on how to answer question
            images ([str]): list of paths to images in question
    """
    def __init__(self, id, question, answers, hint="", images=None):
        try:
            answers = [self.create_answer(answer) for answer in answers]
            super(InputQuestion, self).__init__(id, question, exercises.INPUT_QUESTION, answers, hint, images)
        except ValueError as e:
            raise InvalidInputAnswerException(e)



PRODUCTION_DOMAIN = "contentworkshop.learningequality.org"
DEBUG_DOMAIN = "127.0.0.1:8000"

FILE_DIFF_URL = "http://{domain}/api/internal/file_diff"
FILE_UPLOAD_URL = "http://{domain}/api/internal/file_upload"
CREATE_CHANNEL_URL = "http://{domain}/api/internal/create_channel"
OPEN_CHANNEL_URL = "http://{domain}/open_channel/{invitation_id}/{channel_id}"

def file_diff_url(domain):
	return FILE_DIFF_URL.format(domain=domain)

def file_upload_url(domain):
	return FILE_UPLOAD_URL.format(domain=domain)

def create_channel_url(domain):
	return CREATE_CHANNEL_URL.format(domain=domain)

def open_channel_url(invitation, channel, domain):
	return OPEN_CHANNEL_URL.format(domain=domain, invitation_id=invitation, channel_id=channel)
